<?php
/**
 * @file
 * Default theme implementation to display the basic html structure of a single
 * Drupal page.
 *
 * Variables:
 * - $css: An array of CSS files for the current page.
 * - $language: (object) The language the site is being displayed in.
 *   $language->language contains its textual representation.
 *   $language->dir contains the language direction. It will either be 'ltr' or
 *   'rtl'.
 * - $rdf_namespaces: All the RDF namespace prefixes used in the HTML document.
 * - $grddl_profile: A GRDDL profile allowing agents to extract the RDF data.
 * - $head_title: A modified version of the page title, for use in the TITLE
 *   tag.
 * - $head_title_array: (array) An associative array containing the string parts
 *   that were used to generate the $head_title variable, already prepared to be
 *   output as TITLE tag. The key/value pairs may contain one or more of the
 *   following, depending on conditions:
 *   - title: The title of the current page, if any.
 *   - name: The name of the site.
 *   - slogan: The slogan of the site, if any, and if there is no title.
 * - $head: Markup for the HEAD section (including meta tags, keyword tags, and
 *   so on).
 * - $styles: Style tags necessary to import all CSS files for the page.
 * - $scripts: Script tags necessary to load the JavaScript files and settings
 *   for the page.
 * - $page_top: Initial markup from any modules that have altered the
 *   page. This variable should always be output first, before all other dynamic
 *   content.
 * - $page: The rendered page content.
 * - $page_bottom: Final closing markup from any modules that have altered the
 *   page. This variable should always be output last, after all other dynamic
 *   content.
 * - $classes String of classes that can be used to style contextually through
 *   CSS.
 *
 * @see bootstrap_preprocess_html()
 * @see template_preprocess()
 * @see template_preprocess_html()
 * @see template_process()
 *
 * @ingroup themeable
 */
?><!DOCTYPE html>
<html lang="<?php print $language->language; ?>" dir="<?php print $language->dir; ?>"<?php print $rdf_namespaces;?>>
<head profile="<?php print $grddl_profile; ?>">
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
  <meta http-equiv="X-UA-Compatible" content="IE=Edge,chrome=1">  
  <?php print $head; ?>
  <meta property="og:image" content="<?php print $GLOBALS['base_url'] . '/' . drupal_get_path('theme',$GLOBALS['theme']) . "/img/logo-1024.png"; ?>">
  <title><?php print $head_title; ?></title>
  <?php print $styles; ?>
  <!-- HTML5 element support for IE6-8 -->
  <!--[if lt IE 9]>
    <script src="//html5shiv.googlecode.com/svn/trunk/html5.js"></script>
  <![endif]-->
  <?php print $scripts; ?>
</head>
<!-- <body class="<?php print $classes; ?>" <?php print $attributes;?>>-->
<body>
  
  
  <button id="menu-primary-toggle" class="btn btn-primary menu-primary-toggle visible-sm visible-xs" data-menu-toggler="false"><?php print t("Menu")?></button>
  
 
  
  
  <?php print $page_top; ?>
  <?php print $page; ?>
  <?php print $page_bottom; ?>


  <!-- Open source code -->
    <script>
      window.page = window.location.hash || "#about";

      (function($) {  
        
        var resizeLeftMenu = function() {   // Показывает или скрывает левое меню
          if ($(window).width() >= 992) {   // md media query
            $(".sidebar-menu-first").css('visibility','visible');    // Скрываем левое меню
            //console.log("resizeLeftMenu: " + $(window).width() + "visible");
          } else {
            //console.log("resizeLeftMenu: " + $(window).width() + "hide");
             $(".sidebar-menu-first").css('visibility','hidden');  // Показываем левое меню
          }          
        }
        
        var resizeRightMenu = function() {
          if ($(window).width() >= 992) {   // md media query
            $(".sidebar-menu-second").css('visibility','visible');    // Скрываем левое меню
            //console.log("resizeLeftMenu: " + $(window).width() + "visible");
          } else {
            //console.log("resizeLeftMenu: " + $(window).width() + "hide");
             $(".sidebar-menu-second").css('visibility','hidden');  // Показываем левое меню
          } 
        }
        var resizeMenu = function() {
          resizeLeftMenu();
          resizeRightMenu();
        }
        resizeMenu();
        
        
        $("#menu-primary-toggle").click(function(){   // Скрывает и показывает меню на малых экранах xs и sm
          //console.log("click");
          if ($(window).width() >= 992) return;
          var marginTop = 82;   // На сколько надо поднять отображение
          var height = $(".sidebar-menu-first").outerHeight()-10;     // 10 - из-за margin
          var footerHeight = $("footer.footer").outerHeight();
          if ($(".sidebar-menu-first").css('visibility') == 'visible') {      // надо скрыть 
            //console.log("height " + height + " margin " + $(".sidebar-menu-first").css('margin-top'));
            $(".sidebar-menu-first").css({'visibility':'hidden','margin-top':0,'height':(height - marginTop - footerHeight),'z-index':2});            
          } else {                                                    // Надо показать
            //console.log("height " + height + " margin " + $(".sideba-rmenu").css('margin-top'));
            $(".sidebar-menu-first").css({'visibility':'visible','margin-top':-marginTop,'height':(height + marginTop + footerHeight),'z-index':10});
          }
        });
        
        $(".pages").on("scroll", function(event){          
          if ($(".pages").children().offset().top > -4) {  // скролл не используется            
            //console.log($(".pages").children().offset().top);
            $("#menu-primary-toggle").css("z-index",10);
          } else {                                           // проскроллили, тогда top < 0
            $("#menu-primary-toggle").css("z-index",-1);
          }
        });
        

        $(document).ready(function() {
          if (window.page != "#about") {
            $(".menu").find("li[data-target=" + window.page + "]").trigger("click");
          }
        });

        $(window).on("resize", function() {
          $("html, body").height($(window).height());
          var height = $(window).height() - $(".header-panel").outerHeight();
          $(".main").height(height);
          $(".menu").height(height - $("footer.footer").outerHeight() - ($(".toolbar").height() || 0));
          $(".pages").height($(window).height());
                  //console.log("resize");
          resizeMenu();

        }).trigger("resize");

        $(".menu li").click(function() {
          // Menu
          if (!$(this).data("target")) return;
          if ($(this).is(".active")) return;
          $(".menu li").not($(this)).removeClass("active");
          $(".page").not(page).removeClass("active").hide();
          window.page = $(this).data("target");
          var page = $(window.page);
          window.location.hash = window.page;
          $(this).addClass("active");


          page.show();

          var totop = setInterval(function() {
            $(".pages").animate({scrollTop:0}, 0);
          }, 1);

          setTimeout(function() {
            page.addClass("active");
            setTimeout(function() {
              clearInterval(totop);
            }, 1000);
          }, 100);
        });

        function cleanSource(html) {
          var lines = html.split(/\n/);

          lines.shift();
          lines.splice(-1, 1);

          var indentSize = lines[0].length - lines[0].trim().length,
              re = new RegExp(" {" + indentSize + "}");

          lines = lines.map(function(line){
            if (line.match(re)) {
              line = line.substring(indentSize);
            }

            return line;
          });

          lines = lines.join("\n");

          return lines;
        }

        jQuery.material.init();
        jQuery.material.checkbox("input[type='checkbox']");
        /*

        $("#floating-action-btn").click(function() {
          $.get(window.location.href, function(data){
            var html = $(data).find(window.page).html();
            html = cleanSource(html);
            $("#source-modal pre").text(html);
            $("#source-modal").modal();
          });
        });

        
        
        $(function() {
          $.material.init();
          $(".shor").noUiSlider({
            start: 40,
            connect: "lower",
            range: {
              min: 0,
              max: 100
            }
          });

          $(".svert").noUiSlider({
            orientation: "vertical",
            start: 40,
            connect: "lower",
            range: {
              min: 0,
              max: 100
            }
          });
        });
        */
        
      })(jQuery);       
    </script>
  
  
</body>
</html>
